#include <Wire.h>
#include <Adafruit_PN532.h>
#include <Servo.h>

// Definimos los servos y el LED
Servo servo1;
Servo servo2;
Servo servo3;

const int servoPin1 = D10;
const int servoPin2 = D9;
const int servoPin3 = D8;
const int ledPin = D7;

// Definimos los pines del sensor PN532
#define SDA_PIN 4
#define SCL_PIN 5

Adafruit_PN532 nfc(SDA_PIN, SCL_PIN);

char comando = ' ';
bool tarjetaDetectada = false;

void setup() {
  // Iniciamos el monitor serial
  Serial.begin(115200);
  while (!Serial) {
    ; // Esperar a que el puerto serial se conecte. Solo necesario para puertos serie nativos
  }
  Serial.println("Iniciando...");

  // Asignamos los pines de los servos
  servo1.attach(servoPin1);
  servo2.attach(servoPin2);
  servo3.attach(servoPin3);
  Serial.println("Servos conectados.");

  // Inicializamos los servos en la posición inicial
  servo1.write(0);
  servo2.write(0);
  servo3.write(0);

  // Configuramos el pin del LED
  pinMode(ledPin, OUTPUT);
  Serial.println("LED configurado.");

  // Iniciamos la comunicación I2C usando los pines por defecto
  Wire.begin();
  Serial.println("Comunicación I2C iniciada.");

  // Iniciamos el sensor PN532
  nfc.begin();
  Serial.println("Iniciando PN532...");
  uint32_t versiondata = nfc.getFirmwareVersion();
  if (!versiondata) {
    Serial.println("No se encontró el PN53x. Check hardware connection.");
    while (1) {
      ; // Hacer un loop infinito para detener la ejecución
    }
  }
  Serial.print("Firmware encontrado: ");
  Serial.print((versiondata>>24) & 0xFF, HEX); 
  Serial.print('.'); 
  Serial.print((versiondata>>16) & 0xFF, HEX); 
  Serial.print('.'); 
  Serial.print((versiondata>>8) & 0xFF, HEX); 
  Serial.println((versiondata>>0) & 0xFF, HEX);
  
  Serial.println("PN532 encontrado!");


}

void loop() {
  // Leer el comando desde la interfaz
  if (Serial.available() > 0) {
    comando = Serial.read();
    Serial.print("Comando recibido: ");
    Serial.println(comando);
  }

  uint8_t success;
  uint8_t uid[] = {0, 0, 0, 0, 0, 0, 0}; // Buffer para almacenar el UID de la tarjeta
  uint8_t uidLength;
  // Configurar el sensor para leer RFID
  nfc.SAMConfig();
  Serial.println("Esperando tarjeta...");
  // Esperar hasta que se detecte una tarjeta
  success = nfc.readPassiveTargetID(PN532_MIFARE_ISO14443A, uid, &uidLength);

  if (success) {
    if (!tarjetaDetectada) {
      tarjetaDetectada = true;
      Serial.println("Tarjeta detectada!");
      Serial.print("UID: ");
      for (int i = 0; i < uidLength; i++) {
        Serial.print(" 0x");
        Serial.print(uid[i], HEX);
      }
      Serial.println();
    }

    // Ejecutar comando basado en el botón presionado
    if (comando != ' ') {
      ejecutarComando(comando);
      comando = ' '; // Resetear comando después de ejecutarlo
    }
  } else {
    if (tarjetaDetectada) {
      tarjetaDetectada = false;
      Serial.println("Tarjeta removida, esperando nueva tarjeta...");
    }
  }

  delay(500); // Pequeño retraso antes de volver a comprobar
}

void ejecutarComando(char comando) {
  switch (comando) {
    case 'A':
      // Movemos los servos a la posición A (ejemplo: diferentes ángulos)
      
      digitalWrite(ledPin, HIGH);
      moverServos(170, 60, 90);
      digitalWrite(ledPin, LOW);
      break;

    case 'B':
      // Movemos los servos a la posición B (ejemplo: diferentes ángulos)
      
      digitalWrite(ledPin, HIGH);
      moverServos(45, 90, 135);
      digitalWrite(ledPin, LOW);
      break;
    case 'C':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(72, 160, 145);
      digitalWrite(ledPin, LOW);
      break;

    case 'D':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(160, 120, 90);
      digitalWrite(ledPin, LOW);
      break;

    case 'E':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(30, 120, 180);
      digitalWrite(ledPin, LOW);
      break;

    case 'F':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(60, 90, 180);
      digitalWrite(ledPin, LOW);
      break;

    case 'G':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(110, 90, 60);
      digitalWrite(ledPin, LOW);
      break;

    case 'H':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(60, 120, 30);
      digitalWrite(ledPin, LOW);
      break;

    case 'I':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(60, 120, 60);
      digitalWrite(ledPin, LOW);
      break;

    case 'J':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(60, 120, 90);
      digitalWrite(ledPin, LOW);
      break;

    case 'K':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(22, 160, 165);
      digitalWrite(ledPin, LOW);
      break;

    case 'L':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(60, 120, 150);
      digitalWrite(ledPin, LOW);
      break;

    case 'M':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(60, 120, 180);
      digitalWrite(ledPin, LOW);
      break;

    case 'N':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(30, 120, 60);
      digitalWrite(ledPin, LOW);
      break;

    case 'O':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(120, 60, 180);
      digitalWrite(ledPin, LOW);
      break;

    case 'P':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(30, 90, 120);
      digitalWrite(ledPin, LOW);
      break;

    case 'Q':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(45, 120, 10);
      digitalWrite(ledPin, LOW);
      break;

    case 'R':
      // Movemos los servos a la posición C (ejemplo: diferentes ángulos)
       
      digitalWrite(ledPin, HIGH);
      moverServos(30, 45, 120);
      digitalWrite(ledPin, LOW);
      break;

    default:
      Serial.println("Comando desconocido");
      break;
  }
}

void moverServos(int angulo1, int angulo2, int angulo3) {
  servo1.write(angulo1);
  servo2.write(angulo2);
  servo3.write(angulo3);

  delay(3000); // Mantener el estado por 2 segundos (ajustar según sea necesario)

  // Regresamos los servos a la posición inicial
  Serial.println("Regresando servos a 0 grados");
  servo1.write(0);
  servo2.write(0);
  servo3.write(0);
}